@@ -1,6 +1,6 @@
 package com.carrotsearch.examples.randomizedrunner;
 
-import junit.framework.Assert;
+import org.junit.Assert;
 
 import org.junit.Test;
 import org.junit.runner.notification.RunListener;
