@@ -1,6 +1,6 @@
 package com.carrotsearch.examples.randomizedrunner.reports;
 
-import junit.framework.Assert;
+import org.junit.Assert;
 
 import org.junit.BeforeClass;
 import org.junit.Test;
