@@ -41,17 +41,33 @@ class LineFlushingWriter extends Writer {
   
   @Override
   public Writer append(char c) throws IOException {
-    throw new UnsupportedOperationException();
+    w.append(c);
+    if (c == '\n') flush();
+    return this;
   }
   
   @Override
   public Writer append(CharSequence csq) throws IOException {
-    throw new UnsupportedOperationException();
+    w.append(csq);
+    for (int i = 0, len = csq.length(); i < len; i++) {
+      if (csq.charAt(i) == '\n') {
+        flush();
+        break;
+      }
+    }
+    return this;
   }
 
   @Override
   public Writer append(CharSequence csq, int start, int end) throws IOException {
-    throw new UnsupportedOperationException();
+    w.append(csq, start, end);
+    for (int i = start; i < end; i++) {
+      if (csq.charAt(i) == '\n') {
+        flush();
+        break;
+      }
+    }
+    return this;
   }
 
   @Override
