@@ -1,16 +1,15 @@
 package com.carrotsearch.ant.tasks.junit4.listeners;
 
-import static com.carrotsearch.ant.tasks.junit4.FormattingUtils.formatDescription;
-import static com.carrotsearch.ant.tasks.junit4.FormattingUtils.formatDurationInSeconds;
-import static com.carrotsearch.ant.tasks.junit4.FormattingUtils.formatTime;
-import static com.carrotsearch.ant.tasks.junit4.FormattingUtils.formatTimestamp;
+import static com.carrotsearch.ant.tasks.junit4.FormattingUtils.*;
 
 import java.io.File;
 import java.io.IOException;
 import java.io.OutputStreamWriter;
+import java.io.Reader;
 import java.io.StringWriter;
 import java.io.Writer;
 import java.nio.charset.Charset;
+import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.EnumMap;
 import java.util.HashSet;
@@ -20,7 +19,6 @@ import java.util.Locale;
 import java.util.Set;
 import java.util.concurrent.atomic.AtomicInteger;
 
-import org.apache.commons.io.output.WriterOutputStream;
 import org.apache.tools.ant.BuildException;
 import org.apache.tools.ant.Project;
 import org.junit.runner.Description;
@@ -40,15 +38,18 @@ import com.carrotsearch.ant.tasks.junit4.events.aggregated.AggregatedSuiteStarte
 import com.carrotsearch.ant.tasks.junit4.events.aggregated.AggregatedTestResultEvent;
 import com.carrotsearch.ant.tasks.junit4.events.aggregated.ChildBootstrap;
 import com.carrotsearch.ant.tasks.junit4.events.aggregated.HeartBeatEvent;
+import com.carrotsearch.ant.tasks.junit4.events.aggregated.JvmOutputEvent;
 import com.carrotsearch.ant.tasks.junit4.events.aggregated.PartialOutputEvent;
 import com.carrotsearch.ant.tasks.junit4.events.aggregated.TestStatus;
 import com.carrotsearch.ant.tasks.junit4.events.mirrors.FailureMirror;
+import com.carrotsearch.randomizedtesting.WriterOutputStream;
+import com.carrotsearch.randomizedtesting.annotations.SuppressForbidden;
 import com.google.common.base.Charsets;
+import com.google.common.base.MoreObjects;
 import com.google.common.base.Strings;
-import com.google.common.collect.Lists;
-import com.google.common.collect.Maps;
 import com.google.common.eventbus.Subscribe;
 import com.google.common.io.CharSink;
+import com.google.common.io.CharStreams;
 import com.google.common.io.Closeables;
 import com.google.common.io.FileWriteMode;
 import com.google.common.io.Files;
@@ -107,7 +108,7 @@ public class TextReport implements AggregatedEventListener {
    */
   private static EnumMap<TestStatus, String> statusNames;
   static {
-    statusNames = Maps.newEnumMap(TestStatus.class);
+    statusNames = new EnumMap<>(TestStatus.class);
     for (TestStatus s : TestStatus.values()) {
       statusNames.put(s,
           s == TestStatus.IGNORED_ASSUMPTION
@@ -139,7 +140,7 @@ public class TextReport implements AggregatedEventListener {
    * Initialize {@link #displayStatus}.
    */
   {
-    displayStatus = Maps.newEnumMap(TestStatus.class);
+    displayStatus = new EnumMap<>(TestStatus.class);
     for (TestStatus s : TestStatus.values()) {
       displayStatus.put(s, true);
     }
@@ -201,13 +202,15 @@ public class TextReport implements AggregatedEventListener {
   private int showNumFailuresAtEnd = 3;
   
   /** A list of failed tests, if to be displayed at the end. */
-  private List<Description> failedTests = Lists.newArrayList();
+  private List<Description> failedTests = new ArrayList<>();
 
   /** Stack trace filters. */
-  private List<StackTraceFilter> stackFilters = Lists.newArrayList();
+  private List<StackTraceFilter> stackFilters = new ArrayList<>();
 
   private int totalSuites;
+  private AtomicInteger totalErrors = new AtomicInteger();
   private AtomicInteger suitesCompleted = new AtomicInteger();
+  private String seed;
 
   public void setShowStatusError(boolean showStatus)   { displayStatus.put(TestStatus.ERROR, showStatus); }
   public void setShowStatusFailure(boolean showStatus) { displayStatus.put(TestStatus.FAILURE, showStatus); }
@@ -267,7 +270,7 @@ public class TextReport implements AggregatedEventListener {
    */
   public void setShowOutput(String mode) {
     try {
-      this.outputMode = OutputMode.valueOf(mode.toUpperCase());
+      this.outputMode = OutputMode.valueOf(mode.toUpperCase(Locale.ROOT));
     } catch (IllegalArgumentException e) {
       throw new IllegalArgumentException("showOutput accepts any of: "
           + Arrays.toString(OutputMode.values()) + ", value is not valid: " + mode);
@@ -321,7 +324,10 @@ public class TextReport implements AggregatedEventListener {
    * Initialization by container task {@link JUnit4}.
    */
   @Override
+  @SuppressForbidden("legitimate sysstreams.")
   public void setOuter(JUnit4 task) {
+    this.seed = task.getSeed();
+
     if (outputFile != null) {
       try {
         Files.createParentDirs(outputFile);
@@ -388,7 +394,7 @@ public class TextReport implements AggregatedEventListener {
     if (showNumFailuresAtEnd > 0 && !failedTests.isEmpty()) {
       List<Description> sublist = this.failedTests; 
       StringBuilder b = new StringBuilder();
-      b.append("\nTests with failures");
+      b.append("\nTests with failures [seed: ").append(seed).append("]");
       if (sublist.size() > showNumFailuresAtEnd) {
         sublist = sublist.subList(0, showNumFailuresAtEnd);
         b.append(" (first " + showNumFailuresAtEnd + " out of " + failedTests.size() + ")");
@@ -435,6 +441,18 @@ public class TextReport implements AggregatedEventListener {
     }
   }
 
+  @Subscribe
+  public void onJvmOutput(JvmOutputEvent e) throws IOException {
+    final String id = Integer.toString(e.getSlave().id);
+    output.append(">>> JVM J").append(id)
+          .append(" emitted unexpected output (verbatim) ----\n");
+
+    try (Reader r = Files.newReader(e.getJvmOutputFile(), e.getSlave().getCharset())) {
+      CharStreams.copy(r, output);
+    }
+    output.append("<<< JVM J" + id + ": EOF ----\n");    
+  }
+
   @Subscribe
   public void onTestResult(AggregatedTestResultEvent e) throws IOException {
     if (isPassthrough() && displayStatus.get(e.getStatus())) {
@@ -481,7 +499,7 @@ public class TextReport implements AggregatedEventListener {
   }
 
   private void emitBufferedEvents(AggregatedSuiteResultEvent e) throws IOException {
-    final IdentityHashMap<TestFinishedEvent,AggregatedTestResultEvent> eventMap = Maps.newIdentityHashMap();
+    final IdentityHashMap<TestFinishedEvent,AggregatedTestResultEvent> eventMap = new IdentityHashMap<>();
     for (AggregatedTestResultEvent tre : e.getTests()) {
       eventMap.put(tre.getTestFinishedEvent(), tre);
     }
@@ -551,10 +569,12 @@ public class TextReport implements AggregatedEventListener {
     assert showSuiteSummary;
 
     final StringBuilder b = new StringBuilder();
-    b.append(String.format(Locale.ENGLISH, "%sCompleted [%d/%d]%s in %.2fs, ",
+    final int totalErrors = this.totalErrors.addAndGet(e.isSuccessful() ? 0 : 1);
+    b.append(String.format(Locale.ROOT, "%sCompleted [%d/%d%s]%s in %.2fs, ",
         shortTimestamp(e.getStartTimestamp() + e.getExecutionTime()),
         suitesCompleted,
         totalSuites,
+        totalErrors == 0 ? "" : " (" + totalErrors + "!)",
         e.getSlave().slaves > 1 ? " on J" + e.getSlave().id : "",
         e.getExecutionTime() / 1000.0d));
     b.append(e.getTests().size()).append(Pluralize.pluralize(e.getTests().size(), " test"));
@@ -592,7 +612,7 @@ public class TextReport implements AggregatedEventListener {
     line.append(Strings.padEnd(statusNames.get(status), 8, ' '));
     line.append(formatDurationInSeconds(timeMillis));
     if (forkedJvmCount > 1) {
-      line.append(String.format(Locale.ENGLISH, jvmIdFormat, result.getSlave().id));
+      line.append(String.format(Locale.ROOT, jvmIdFormat, result.getSlave().id));
     }
     line.append(" | ");
 
@@ -621,11 +641,11 @@ public class TextReport implements AggregatedEventListener {
         for (FailureMirror fm : failures) {
           count++;
             if (fm.isAssumptionViolation()) {
-                pos.write(String.format(Locale.ENGLISH, 
+                pos.write(String.format(Locale.ROOT, 
                     "Assumption #%d: %s",
-                    count, com.google.common.base.Objects.firstNonNull(fm.getMessage(), "(no message)")));
+                    count, MoreObjects.firstNonNull(fm.getMessage(), "(no message)")));
             } else {
-                pos.write(String.format(Locale.ENGLISH, 
+                pos.write(String.format(Locale.ROOT, 
                     "Throwable #%d: %s",
                     count,
                     showStackTraces ? filterStackTrace(fm.getTrace()) : fm.getThrowableString()));
