@@ -12,6 +12,9 @@ import java.io.OutputStreamWriter;
 import java.io.PrintStream;
 import java.io.RandomAccessFile;
 import java.io.Writer;
+import java.nio.file.Files;
+import java.nio.file.Path;
+import java.nio.file.Paths;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.util.ArrayDeque;
@@ -21,6 +24,7 @@ import java.util.Collections;
 import java.util.Iterator;
 import java.util.List;
 
+import com.carrotsearch.randomizedtesting.RandomizedRunner;
 import org.junit.runner.Description;
 import org.junit.runner.Request;
 import org.junit.runner.Result;
@@ -38,9 +42,9 @@ import com.carrotsearch.ant.tasks.junit4.events.Serializer;
 import com.carrotsearch.ant.tasks.junit4.events.SuiteFailureEvent;
 import com.carrotsearch.randomizedtesting.MethodGlobFilter;
 import com.carrotsearch.randomizedtesting.SysGlobals;
+import com.carrotsearch.randomizedtesting.annotations.SuppressForbidden;
 import com.google.common.base.Strings;
 import com.google.common.collect.Iterators;
-import com.google.common.collect.Lists;
 
 /**
  * A slave process running the actual tests on the target JVM.
@@ -102,6 +106,11 @@ public class SlaveMain {
    */
   public static final String OPTION_DEBUGSTREAM = "-debug";
 
+  /**
+   * User-defined RunListener classes.
+   */
+  public static final String OPTION_RUN_LISTENERS = "-runListeners";
+
   /**
    * Fire a runner failure after startup to verify messages
    * are propagated properly. Not really useful in practice...
@@ -109,6 +118,13 @@ public class SlaveMain {
   public static final String SYSPROP_FIRERUNNERFAILURE =
       SlaveMain.class.getName() + ".fireRunnerFailure";
 
+  /**
+   * Delay the initial bootstrap event from the forked JVM 
+   * (used in tests).
+   */
+  public static final String SYSPROP_FORKEDJVM_DELAY_MS =
+      "junit4.tests.internal.initialDelayMs";
+
   /**
    * Event sink.
    */
@@ -122,7 +138,10 @@ public class SlaveMain {
 
   /** Debug stream to flush progress information to. */
   private File debugMessagesFile;
-  
+
+  /** List of RunListener classes */
+  private String runListeners;
+
   /** 
    * Multiplex calls to System streams to both event stream
    * and the original streams?
@@ -238,12 +257,22 @@ public class SlaveMain {
         try {
           Runner runner = request.getRunner();
           methodFilter.apply(runner);
-  
+
+          // New RunListener instances should be added per class and then removed from the RunNotifier
+          ArrayList<RunListener> runListenerInstances = instantiateRunListeners();
+          for (RunListener runListener : runListenerInstances) {
+            fNotifier.addListener(runListener);
+          }
+
           fNotifier.fireTestRunStarted(runner.getDescription());
           debug(debug, "Runner.run(" + clName + ")");
           runner.run(fNotifier);
           debug(debug, "Runner.done(" + clName + ")");
           fNotifier.fireTestRunFinished(result);
+
+          for (RunListener runListener : runListenerInstances) {
+            fNotifier.removeListener(runListener);
+          }
         } catch (NoTestsRemainException e) {
           // Don't complain if all methods have been filtered out. 
           // I don't understand the reason why this exception has been
@@ -301,7 +330,8 @@ public class SlaveMain {
       boolean flushFrequently = false;
       File  eventsFile = null;
       boolean suitesOnStdin = false;
-      List<String> testClasses = Lists.newArrayList();
+      List<String> testClasses = new ArrayList<>();
+      String runListeners = null;
 
       while (!args.isEmpty()) {
         String option = args.pop();
@@ -318,6 +348,8 @@ public class SlaveMain {
             raf.setLength(0);
             raf.close();
           }
+        } else if (option.equals(OPTION_RUN_LISTENERS)) {
+          runListeners = args.pop();
         } else if (option.startsWith(OPTION_DEBUGSTREAM)) {
           debugStream = true;
         } else if (option.startsWith("@")) {
@@ -333,6 +365,12 @@ public class SlaveMain {
       if (eventsFile == null) {
         throw new IOException("You must specify communication channel for events.");
       }
+
+      // Delay the forked JVM a bit (for tests).
+      if (System.getProperty(SYSPROP_FORKEDJVM_DELAY_MS) != null) {
+        Thread.sleep(Integer.parseInt(System.getProperty(SYSPROP_FORKEDJVM_DELAY_MS)));
+      }
+      
       // Send bootstrap package.
       serializer = new Serializer(new EventsOutputStream(eventsFile))
         .serialize(new BootstrapEvent())
@@ -344,6 +382,7 @@ public class SlaveMain {
       final SlaveMain main = new SlaveMain(serializer);
       main.flushFrequently = flushFrequently;
       main.debugMessagesFile = debugStream ? new File(eventsFile.getAbsolutePath() + ".debug"): null;
+      main.runListeners = runListeners;
 
       final Iterator<String> stdInput;
       if (suitesOnStdin) { 
@@ -389,8 +428,8 @@ public class SlaveMain {
    * here we're neck deep in sh*t anyway (OOMs all over).
    */
   private static void tryWaitingForGC() {
-    // TODO: we could try to preallocate memory mx bean and count collections.
-    // there is no guarantee it doesn't allocate stuff too though. 
+    // We could try to preallocate memory mx bean and count collections...
+    // there is no guarantee it doesn't allocate stuff too though.
     final long timeout = System.currentTimeMillis() + 2000;
     while (System.currentTimeMillis() < timeout) {
       System.gc(); 
@@ -428,6 +467,7 @@ public class SlaveMain {
   /**
    * Redirect standard streams so that the output can be passed to listeners.
    */
+  @SuppressForbidden("legitimate sysstreams.")
   private static void redirectStreams(final Serializer serializer, final boolean flushFrequently) {
     final PrintStream origSysOut = System.out;
     final PrintStream origSysErr = System.err;
@@ -435,6 +475,7 @@ public class SlaveMain {
     // Set warnings stream to System.err.
     warnings = System.err;
     AccessController.doPrivileged(new PrivilegedAction<Void>() {
+      @SuppressForbidden("legitimate PrintStream with default charset.")
       @Override
       public Void run() {
         System.setOut(new PrintStream(new BufferedOutputStream(new ChunkedStream() {
@@ -466,6 +507,7 @@ public class SlaveMain {
   /**
    * Warning emitter. Uses whatever alternative non-event communication channel is.
    */
+  @SuppressForbidden("legitimate sysstreams.")
   public static void warn(String message, Throwable t) {
     PrintStream w = (warnings == null ? System.err : warnings);
     try {
@@ -493,4 +535,19 @@ public class SlaveMain {
       w.println("ERROR: Couldn't even serialize a warning.");
     }
   }
+
+  /**
+   * Generates JUnit 4 RunListener instances for any user defined RunListeners
+   */
+  private ArrayList<RunListener> instantiateRunListeners() throws Exception {
+    ArrayList<RunListener> instances = new ArrayList<>();
+
+    if (runListeners != null) {
+      for (String className : Arrays.asList(runListeners.split(","))) {
+        instances.add((RunListener) this.instantiate(className).newInstance());
+      }
+    }
+
+    return instances;
+  }
 }
