@@ -1,13 +1,17 @@
 package com.carrotsearch.maven.plugins.junit4;
 
+import static com.google.common.base.MoreObjects.*;
+
 import java.io.File;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.io.StringReader;
 import java.io.StringWriter;
+import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
+import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -48,10 +52,7 @@ import org.dom4j.io.XMLWriter;
 import com.carrotsearch.ant.tasks.junit4.JUnit4;
 import com.carrotsearch.ant.tasks.junit4.listeners.TextReport;
 import com.carrotsearch.randomizedtesting.RandomizedRunner;
-import com.google.common.base.Objects;
 import com.google.common.base.Strings;
-import com.google.common.collect.Lists;
-import com.google.common.collect.Sets;
 import com.google.common.io.Closer;
 
 /**
@@ -153,14 +154,22 @@ public class JUnit4Mojo extends AbstractMojo {
   private boolean haltOnFailure = JUnit4.DEFAULT_HALT_ON_FAILURE;
 
   /**
-   * If set to <code>true</code> each slave JVM gets a separate working directory
-   * under whatever is set in <code>dir</code>. The directory naming for each slave
-   * follows: "S<i>num</i>", where <i>num</i> is slave's number. Directories are created
-   * automatically and removed unless <code>leaveTemporary</code> is set to
-   * <code>true</code>.
+   * If set to <code>true</code> each forked JVM gets a separate working directory
+   * under whatever is set in <code>dir</code>. The directory naming for each JVM
+   * follows: "J<i>num</i>", where <i>num</i> is the forked JVM's number. 
+   * Directories are created automatically and removed unless <code>leaveTemporary</code> 
+   * is set to <code>true</code>.
    */
+  @Parameter(defaultValue = "true")
   private boolean isolateWorkingDirectories = JUnit4.DEFAULT_ISOLATE_WORKING_DIRECTORIES;
 
+  /**
+   * Sets the action performed when current work directory for a forked JVM is not empty
+   * and <code>isolateWorkingDirectories</code> is set to true.
+   */
+  @Parameter(defaultValue = "fail")
+  private String onNonEmptyWorkDirectory = JUnit4.DEFAULT_NON_EMPTY_WORKDIR_ACTION.name();
+
   /**
    * If set to true, any sysout and syserr calls will be written to original
    * output and error streams (and in effect will appear as "jvm output". By default
@@ -275,9 +284,8 @@ public class JUnit4Mojo extends AbstractMojo {
    * This parameter adds a listener emitting surefire-compatible XMLs if no other listeners
    * are added. If there are any configured listeners, this parameter is omitted (you can
    * add a maven-compatible listener manually).
-   * 
-   * @parameter default-value="${project.build.directory}/surefire-reports"
    */
+  @Parameter(defaultValue = "${project.build.directory}/surefire-reports")
   private File surefireReportsDirectory;
 
   /**
@@ -317,6 +325,13 @@ public class JUnit4Mojo extends AbstractMojo {
   @Parameter
   private PlexusConfiguration listeners;
 
+  /**
+   * Raw runListeners configuration. Same XML as for ANT.
+   */
+  @Parameter
+  private PlexusConfiguration runListeners;
+
+
   /**
    * Raw assertions configuration. Same XML as for ANT.
    */
@@ -349,8 +364,6 @@ public class JUnit4Mojo extends AbstractMojo {
   /**
    * Set this to "true" to skip running tests, but still compile them. Its use
    * is NOT RECOMMENDED, but quite convenient on occasion.
-   * 
-   * @parameter default-value="false" property="skipTests"
    */
   @Parameter(
       property = "skipTests",
@@ -593,7 +606,8 @@ public class JUnit4Mojo extends AbstractMojo {
     if (newEnvironment != null) junit4.addAttribute("newEnvironment", newEnvironment.toString());
     if (ifNoTests != null) junit4.addAttribute("ifNoTests", ifNoTests);
     if (statsPropertyPrefix != null) junit4.addAttribute("statsPropertyPrefix", statsPropertyPrefix);
-      
+    if (onNonEmptyWorkDirectory != null) junit4.addAttribute("onNonEmptyWorkDirectory", onNonEmptyWorkDirectory);
+
     junit4.addAttribute("shuffleOnSlave", Boolean.toString(shuffleOnSlave));
     junit4.addAttribute("printSummary", Boolean.toString(printSummary));
     junit4.addAttribute("isolateWorkingDirectories", Boolean.toString(isolateWorkingDirectories));
@@ -605,7 +619,7 @@ public class JUnit4Mojo extends AbstractMojo {
 
 
     // JVM args.
-    for (String jvmArg : Objects.firstNonNull(jvmArgs, EMPTY_STRING_ARRAY)) {
+    for (String jvmArg : firstNonNull(jvmArgs, EMPTY_STRING_ARRAY)) {
       junit4.addElement("jvmarg").addAttribute("value", jvmArg);
     }
     
@@ -615,7 +629,7 @@ public class JUnit4Mojo extends AbstractMojo {
 
     // System properties
     for (Map.Entry<String,String> e : 
-      Objects.firstNonNull(systemProperties, EMPTY_STRING_STRING_MAP).entrySet()) {
+      firstNonNull(systemProperties, EMPTY_STRING_STRING_MAP).entrySet()) {
       Element sysproperty = junit4.addElement("sysproperty");
       sysproperty.addAttribute("key", Strings.nullToEmpty(e.getKey()));
       sysproperty.addAttribute("value", Strings.nullToEmpty(e.getValue()));
@@ -623,7 +637,7 @@ public class JUnit4Mojo extends AbstractMojo {
     
     // Environment variables.
     for (Map.Entry<String,String> e : 
-      Objects.firstNonNull(environmentVariables, EMPTY_STRING_STRING_MAP).entrySet()) {
+      firstNonNull(environmentVariables, EMPTY_STRING_STRING_MAP).entrySet()) {
       Element sysproperty = junit4.addElement("env");
       sysproperty.addAttribute("key", Strings.nullToEmpty(e.getKey()));
       sysproperty.addAttribute("value", Strings.nullToEmpty(e.getValue()));
@@ -659,6 +673,9 @@ public class JUnit4Mojo extends AbstractMojo {
         consoleReport.addAttribute("showSuiteSummary",  "true");
     }
 
+    // Copy over runlisteners
+    appendRawXml(runListeners, junit4);
+
     // Copy over assertions
     appendRawXml(assertions, junit4);
     
@@ -760,7 +777,7 @@ public class JUnit4Mojo extends AbstractMojo {
   private ArtifactResolutionResult resolveArtifact(Artifact artifact, Artifact... filtered) {
     final ArtifactFilter filter;
     if (filtered.length > 0) {
-      List<String> exclusions = Lists.newArrayListWithExpectedSize(filtered.length);
+      List<String> exclusions = new ArrayList<>(filtered.length);
       for (Artifact filteredArtifact : filtered) {
         exclusions.add(filteredArtifact.getGroupId() + ":"
             + filteredArtifact.getArtifactId());
@@ -828,9 +845,8 @@ public class JUnit4Mojo extends AbstractMojo {
     cp.addComment("Additional classpath elements.");
     if (additionalClasspathElements != null && !additionalClasspathElements.isEmpty()) {
       for (String classpathElement : additionalClasspathElements) {
-        if (Strings.isNullOrEmpty(classpathElement)) {
-          cp.addElement("pathelement").addAttribute("location",
-              classpathElement);
+        if (!Strings.isNullOrEmpty(classpathElement)) {
+          cp.addElement("pathelement").addAttribute("location", classpathElement);
         }
       }
     }
@@ -840,7 +856,7 @@ public class JUnit4Mojo extends AbstractMojo {
    * Return a new set containing only the artifacts accepted by the given filter.
    */
   private Set<Artifact> filterArtifacts(Element cp, Set<Artifact> artifacts, ArtifactFilter filter) {
-    Set<Artifact> filteredArtifacts = Sets.newLinkedHashSet();
+    Set<Artifact> filteredArtifacts = new LinkedHashSet<>();
     for (Artifact artifact : artifacts) {
       if (!filter.include(artifact)) {
         filteredArtifacts.add(artifact);
