@@ -1,10 +1,11 @@
 package com.carrotsearch.randomizedtesting;
 
 import java.io.Closeable;
-import java.io.File;
 import java.io.IOException;
 import java.net.ServerSocket;
 import java.nio.charset.Charset;
+import java.nio.file.*;
+import java.nio.file.attribute.BasicFileAttributes;
 import java.text.SimpleDateFormat;
 import java.util.Arrays;
 import java.util.Comparator;
@@ -14,14 +15,21 @@ import java.util.List;
 import java.util.Locale;
 import java.util.Random;
 import java.util.TimeZone;
+import java.util.concurrent.atomic.AtomicInteger;
+import java.util.function.Supplier;
 
-import org.junit.Assert;
 import org.junit.Assume;
+import org.junit.AssumptionViolatedException;
 import org.junit.runner.RunWith;
 
 import com.carrotsearch.randomizedtesting.annotations.Listeners;
 import com.carrotsearch.randomizedtesting.annotations.Nightly;
-import com.carrotsearch.randomizedtesting.generators.*;
+import com.carrotsearch.randomizedtesting.annotations.SuppressForbidden;
+import com.carrotsearch.randomizedtesting.generators.BiasedNumbers;
+import com.carrotsearch.randomizedtesting.generators.RandomBytes;
+import com.carrotsearch.randomizedtesting.generators.RandomNumbers;
+import com.carrotsearch.randomizedtesting.generators.RandomPicks;
+import com.carrotsearch.randomizedtesting.generators.RandomStrings;
 
 /**
  * Common scaffolding for subclassing randomized tests.
@@ -30,7 +38,7 @@ import com.carrotsearch.randomizedtesting.generators.*;
  * @see RandomizedContext
  */
 @RunWith(RandomizedRunner.class)
-public class RandomizedTest extends Assert {
+public class RandomizedTest {
   /**
    * The global multiplier property (Double).
    * 
@@ -64,13 +72,14 @@ public class RandomizedTest extends Assert {
   }
 
   /**
-   * Returns true if we're running nightly tests.
+   * Returns true if {@link Nightly} test group is enabled.
+   * 
    * @see Nightly
    */
   public static boolean isNightly() {
     return getContext().isNightly();
   }
-  
+
   /**
    * Shortcut for {@link RandomizedContext#getRandom()}. Even though this method
    * is static, it returns per-thread {@link Random} instance, so no race conditions
@@ -98,14 +107,66 @@ public class RandomizedTest extends Assert {
   public static double  randomGaussian() { return getRandom().nextGaussian(); }
 
   //
-  // Delegates to RandomInts.
+  // Biased value pickers. 
+  //
+  
+  /**
+   * A biased "evil" random float between min and max (inclusive).
+   * 
+   * @see BiasedNumbers#randomFloatBetween(Random, float, float)
+   */
+  public static float  biasedFloatBetween(float min, float max) { return BiasedNumbers.randomFloatBetween(getRandom(), min, max); }
+
+  /**
+   * A biased "evil" random double between min and max (inclusive).
+   * 
+   * @see BiasedNumbers#randomDoubleBetween(Random, double, double)
+   */
+  public static double biasedDoubleBetween(double min, double max) { return BiasedNumbers.randomDoubleBetween(getRandom(), min, max); }
+
+  //
+  // Delegates to RandomBytes.
+  //
+
+  /** 
+   * Returns a byte array with random content.
+   * 
+   * @param length The length of the byte array. Can be zero.
+   * @return Returns a byte array with random content. 
+   */
+  public static byte[] randomBytesOfLength(int length) { 
+    return RandomBytes.randomBytesOfLength(new Random(getRandom().nextLong()), length); 
+  }  
+
+  /** 
+   * Returns a byte array with random content.
+   * 
+   * @param minLength The minimum length of the byte array. Can be zero.
+   * @param maxLength The maximum length of the byte array. Can be zero.
+   * @return Returns a byte array with random content. 
+   */
+  public static byte[] randomBytesOfLength(int minLength, int maxLength) { 
+    return RandomBytes.randomBytesOfLengthBetween(new Random(getRandom().nextLong()), minLength, maxLength); 
+  }  
+
+  //
+  // Delegates to RandomNumbers.
   //
 
   /** 
    * A random integer from 0..max (inclusive). 
    */
-  public static int randomInt(int max) { 
-    return RandomInts.randomInt(getRandom(), max); 
+  @Deprecated
+  public static int randomInt(int max) {
+    return RandomNumbers.randomIntBetween(getRandom(), 0, max);
+  }
+
+  /** 
+   * A random long from 0..max (inclusive). 
+   */
+  @Deprecated
+  public static long randomLong(long max) {
+    return RandomNumbers.randomLongBetween(getRandom(), 0, max);
   }
 
   /** 
@@ -114,7 +175,7 @@ public class RandomizedTest extends Assert {
    * @see #scaledRandomIntBetween(int, int)
    */
   public static int randomIntBetween(int min, int max) {
-    return RandomInts.randomIntBetween(getRandom(), min, max);
+    return RandomNumbers.randomIntBetween(getRandom(), min, max);
   }
 
   /** 
@@ -126,6 +187,20 @@ public class RandomizedTest extends Assert {
     return randomIntBetween(min, max);
   }
 
+  /** 
+   * A random long from <code>min</code> to <code>max</code> (inclusive).
+   */
+  public static long randomLongBetween(long min, long max) {
+    return RandomNumbers.randomLongBetween(getRandom(), min, max);
+  }
+
+  /** 
+   * An alias for {@link #randomLongBetween}. 
+   */
+  public static long between(long min, long max) {
+    return randomLongBetween(min, max);
+  }
+
   /** 
    * Returns a random value greater or equal to <code>min</code>. The value
    * picked is affected by {@link #isNightly()} and {@link #multiplier()}.
@@ -186,6 +261,14 @@ public class RandomizedTest extends Assert {
     return RandomPicks.randomFrom(getRandom(), list);
   }
 
+  public static byte randomFrom(byte [] array)     { return RandomPicks.randomFrom(getRandom(), array); }
+  public static short randomFrom(short [] array)   { return RandomPicks.randomFrom(getRandom(), array); }
+  public static int randomFrom(int [] array)       { return RandomPicks.randomFrom(getRandom(), array); }
+  public static char randomFrom(char [] array)     { return RandomPicks.randomFrom(getRandom(), array); }
+  public static float randomFrom(float [] array)   { return RandomPicks.randomFrom(getRandom(), array); }
+  public static long randomFrom(long [] array)     { return RandomPicks.randomFrom(getRandom(), array); }
+  public static double randomFrom(double [] array) { return RandomPicks.randomFrom(getRandom(), array); }
+
   //
   // "multiplied" or scaled value pickers. These will be affected by global multiplier.
   //
@@ -197,7 +280,6 @@ public class RandomizedTest extends Assert {
    * <p>The default multiplier value is 1.</p>
    *
    * @see #SYSPROP_MULTIPLIER
-   * @see #DEFAULT_MULTIPLIER
    */
   public static double multiplier() {
     checkContext();
@@ -250,75 +332,84 @@ public class RandomizedTest extends Assert {
   /**
    * @see #globalTempDir()
    */
-  private static File globalTempDir;
+  private static Path globalTempDir;
   
-  /**
-   * Subfolders under {@link #globalTempDir} are created synchronously, so we don't need
-   * to mangle filenames.
-   */
-  private static int  tempSubFileNameCount;
+  /** */
+  private static AtomicInteger tempSubFileNameCount = new AtomicInteger(0);
 
   /**
    * Global temporary directory created for the duration of this class's lifespan. If
    * multiple class loaders are used, there may be more global temp dirs, but it
    * shouldn't really be the case in practice.
    */
-  public static File globalTempDir() {
+  public static Path globalTempDir() throws IOException {
     checkContext();
     synchronized (RandomizedTest.class) {
       if (globalTempDir == null) {
         String tempDirPath = System.getProperty("java.io.tmpdir");
         if (tempDirPath == null) 
-          throw new Error("No property java.io.tmpdir?");
+          throw new IOException("No property java.io.tmpdir?");
 
-        File tempDir = new File(tempDirPath);
-        if (!tempDir.isDirectory() || !tempDir.canWrite()) {
-          throw new Error("Temporary folder not accessible: "
-              + tempDir.getAbsolutePath());
+        Path tempDir = Paths.get(tempDirPath);
+        if (!Files.isDirectory(tempDir) || !Files.isWritable(tempDir)) {
+          throw new IOException("Temporary folder not accessible: " + tempDir.toAbsolutePath());
         }
 
-        SimpleDateFormat tsFormat = new SimpleDateFormat("'tests-'yyyyMMddHHmmss'-'SSS", Locale.ROOT);
-        int retries = 10;
-        do {
-          String dirName = tsFormat.format(new Date());
-          final File tmpFolder = new File(tempDir, dirName);
-          // I assume mkdir is filesystem-atomic and only succeeds if the 
-          // directory didn't previously exist?
-          if (tmpFolder.mkdir()) {
-            globalTempDir = tmpFolder;
-            Runtime.getRuntime().addShutdownHook(new Thread() {
-              public void run() {
-                try {
-                  forceDeleteRecursively(globalTempDir);
-                } catch (IOException e) {
-                  // Not much else to do but to log and quit.
-                  System.err.println("Error while deleting temporary folder '" +
-                      globalTempDir.getAbsolutePath() +
-                  		"': " + e.getMessage());
-                }
+        globalTempDir = createTemp(tempDir, true, () -> {
+          SimpleDateFormat tsFormat = new SimpleDateFormat("'tests-'yyyyMMddHHmmss'-'SSS", Locale.ROOT);
+          return tsFormat.format(new Date());
+        });
 
-                if (globalTempDir.exists()) {
-                  System.err.println("Could not delete temporary folder entirely: "
-                      + globalTempDir.getAbsolutePath());
-                }                
+        Runtime.getRuntime().addShutdownHook(new Thread() {
+            @SuppressForbidden("Legitimate use of syserr.")
+            public void run() {
+              try {
+                rmDir(globalTempDir);
+              } catch (IOException e) {
+                // Not much else to do but to log and quit.
+                System.err.println("Could not delete global temporary folder: "
+                    + globalTempDir.toAbsolutePath() + ". Cause: ");
+                e.printStackTrace(System.err);
               }
-            });
-            return globalTempDir;
-          }
-        } while (retries-- > 0);
-        throw new RuntimeException("Could not create temporary space in: "
-            + tempDir);
+            }
+          });
       }
       return globalTempDir;
     }
   }
 
+  private static Path createTemp(Path parent, boolean dir, Supplier<String> nameSupplier) throws IOException {
+    int retries = 10;
+    Path target;
+    do {
+      try {
+        Path candidate = parent.resolve(nameSupplier.get());
+        if (dir) {
+          target = Files.createDirectory(candidate);
+        } else {
+          target = Files.createFile(candidate);
+        }
+        break;
+      } catch (FileAlreadyExistsException e) {
+        if (--retries < 0) {
+          throw new IOException("Could not create a unique temporary folder under: " + parent);
+        }
+        try {
+          Thread.sleep(new Random().nextInt(250));
+        } catch (InterruptedException ex) {
+          throw new IOException(ex);
+        }
+      }
+    } while (true);
+    return target;
+  }
+
   /**
    * Creates a new temporary directory for the {@link LifecycleScope#TEST} duration.
    * 
    * @see #globalTempDir()
    */
-  public File newTempDir() {
+  public Path newTempDir() throws IOException {
     return newTempDir(LifecycleScope.TEST);
   }
 
@@ -326,12 +417,10 @@ public class RandomizedTest extends Assert {
    * Creates a temporary directory, deleted after the given lifecycle phase. 
    * Temporary directory is created relative to a globally picked temporary directory.
    */
-  public static File newTempDir(LifecycleScope scope) {
+  public static Path newTempDir(LifecycleScope scope) throws IOException {
     checkContext();
     synchronized (RandomizedTest.class) {
-      File tempDir = new File(globalTempDir(), nextTempName());
-      if (!tempDir.mkdir()) throw new RuntimeException("Could not create temporary folder: "
-          + tempDir.getAbsolutePath());
+      Path tempDir = createTemp(globalTempDir(), true, RandomizedTest::nextTempName);
       getContext().closeAtEnd(new TempPathResource(tempDir), scope);
       return tempDir;
     }
@@ -360,7 +449,7 @@ public class RandomizedTest extends Assert {
   /**
    * Creates a new temporary file for the {@link LifecycleScope#TEST} duration.
    */
-  public File newTempFile() {
+  public Path newTempFile() throws IOException {
     return newTempFile(LifecycleScope.TEST);
   }
 
@@ -368,43 +457,55 @@ public class RandomizedTest extends Assert {
    * Creates a new temporary file deleted after the given lifecycle phase completes.
    * The file is physically created on disk, but is not locked or opened.
    */
-  public static File newTempFile(LifecycleScope scope) {
+  public static Path newTempFile(LifecycleScope scope) throws IOException {
     checkContext();
     synchronized (RandomizedTest.class) {
-      File tempFile = new File(globalTempDir(), nextTempName());
-      try {
-        if (!tempFile.createNewFile()) 
-          throw new RuntimeException("Could not create temporary file: " 
-              + tempFile.getAbsolutePath());
-      } catch (IOException e) {
-        throw new RuntimeException("Could not create temporary file: " 
-            + tempFile.getAbsolutePath(), e);
-      }
+      Path tempFile = createTemp(globalTempDir(), false, RandomizedTest::nextTempName);
       getContext().closeAtEnd(new TempPathResource(tempFile), scope);
       return tempFile;
     }
   }
 
   /** Next temporary filename. */
-  private static String nextTempName() {
-    return String.format("%04d has-space", tempSubFileNameCount++);
+  protected static String nextTempName() {
+    return String.format(Locale.ROOT, "%04d has-space", tempSubFileNameCount.getAndIncrement());
   }
 
   /**
-   * Recursively delete a folder (or file). This attempts to delete everything that
-   * can be deleted, but possibly can leave things behind if files are locked for example.
+   * Recursively delete a folder. Throws an exception if any failure occurs.
+   * 
+   * @param path Path to the folder to be (recursively) deleted. The folder must
+   *             exist.
    */
-  static void forceDeleteRecursively(File fileOrDir) throws IOException {
-    if (fileOrDir.isDirectory()) {
-      // We are not checking for symlinks here!
-      for (File f : fileOrDir.listFiles()) {
-        forceDeleteRecursively(f);
-      }
+  public static void rmDir(Path path) throws IOException {
+    if (!Files.isDirectory(path)) {
+      throw new IOException("Not a folder: " + path);
     }
 
-    if (!fileOrDir.delete()) {
-      RandomizedRunner.logger.warning("Could not delete: "
-          + fileOrDir.getAbsolutePath());
+    try {
+      Files.walkFileTree(path, new SimpleFileVisitor<Path>() {
+        @Override
+        public FileVisitResult postVisitDirectory(Path dir, IOException iterationError) throws IOException {
+          if (iterationError != null) {
+            throw iterationError;
+          }
+          Files.delete(dir);
+          return FileVisitResult.CONTINUE;
+        }
+  
+        @Override
+        public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException {
+          Files.delete(file);
+          return FileVisitResult.CONTINUE;
+        }
+  
+        @Override
+        public FileVisitResult visitFileFailed(Path file, IOException e) throws IOException {
+          throw e;
+        }
+      });
+    } catch (IOException e) {
+      throw new IOException("Could not remove directory: " + path, e);
     }
   }
 
@@ -463,59 +564,107 @@ public class RandomizedTest extends Assert {
   // Characters and strings. Delegates to RandomStrings and that in turn to StringGenerators.
   //
 
-  /** @see StringGenerator#ofCodeUnitsLength(Random, int, int) */
+  /** 
+   * @deprecated Use {@link #randomAsciiLettersOfLengthBetween} instead.  
+   */
+  @Deprecated
   public static String randomAsciiOfLengthBetween(int minCodeUnits, int maxCodeUnits) {
-    return RandomStrings.randomAsciiOfLengthBetween(getRandom(), minCodeUnits,
-        maxCodeUnits);
+    return randomAsciiLettersOfLengthBetween(minCodeUnits, maxCodeUnits);
   }
-  
-  /** @see StringGenerator#ofCodeUnitsLength(Random, int, int) */
+
+  /** 
+   * @deprecated Use {@link #randomAsciiLettersOfLength} instead.  
+   */
+  @Deprecated
   public static String randomAsciiOfLength(int codeUnits) {
-    return RandomStrings.randomAsciiOfLength(getRandom(), codeUnits);
+    return randomAsciiLettersOfLength(codeUnits);
   }
-  
-  /** @see StringGenerator#ofCodeUnitsLength(Random, int, int) */
+
+  /**
+   * @see RandomStrings#randomAsciiLettersOfLengthBetween
+   */
+  public static String randomAsciiLettersOfLengthBetween(int minLetters, int maxLetters) {
+    return RandomStrings.randomAsciiLettersOfLengthBetween(getRandom(), minLetters, maxLetters);
+  }
+
+  /**
+   * @see RandomStrings#randomAsciiLettersOfLength
+   */
+  public static String randomAsciiLettersOfLength(int codeUnits) {
+    return RandomStrings.randomAsciiLettersOfLength(getRandom(), codeUnits);
+  }
+
+  /**
+   * @see RandomStrings#randomAsciiAlphanumOfLengthBetween
+   */
+  public static String randomAsciiAlphanumOfLengthBetween(int minCodeUnits, int maxCodeUnits) {
+    return RandomStrings.randomAsciiAlphanumOfLengthBetween(getRandom(), minCodeUnits, maxCodeUnits);
+  }
+
+  /**
+   * @see RandomStrings#randomAsciiAlphanumOfLength
+   */
+  public static String randomAsciiAlphanumOfLength(int codeUnits) {
+    return RandomStrings.randomAsciiAlphanumOfLength(getRandom(), codeUnits);
+  }
+
+  /**
+   * @see RandomStrings#randomUnicodeOfLengthBetween
+   */
   public static String randomUnicodeOfLengthBetween(int minCodeUnits, int maxCodeUnits) {
     return RandomStrings.randomUnicodeOfLengthBetween(getRandom(),
         minCodeUnits, maxCodeUnits);
   }
-  
-  /** @see StringGenerator#ofCodeUnitsLength(Random, int, int) */
+
+  /**
+   * @see RandomStrings#randomUnicodeOfLength
+   */
   public static String randomUnicodeOfLength(int codeUnits) {
     return RandomStrings.randomUnicodeOfLength(getRandom(), codeUnits);
   }
   
-  /** @see StringGenerator#ofCodePointsLength(Random, int, int) */
+  /**
+   * @see RandomStrings#randomUnicodeOfCodepointLengthBetween
+   */
   public static String randomUnicodeOfCodepointLengthBetween(int minCodePoints, int maxCodePoints) {
     return RandomStrings.randomUnicodeOfCodepointLengthBetween(getRandom(),
         minCodePoints, maxCodePoints);
   }
   
-  /** @see StringGenerator#ofCodePointsLength(Random, int, int) */
+  /**
+   * @see RandomStrings#randomUnicodeOfCodepointLength
+   */
   public static String randomUnicodeOfCodepointLength(int codePoints) {
-    return RandomStrings
-        .randomUnicodeOfCodepointLength(getRandom(), codePoints);
+    return RandomStrings.randomUnicodeOfCodepointLength(getRandom(), codePoints);
   }
   
-  /** @see StringGenerator#ofCodeUnitsLength(Random, int, int) */
+  /**
+   * @see RandomStrings#randomRealisticUnicodeOfLengthBetween
+   */
   public static String randomRealisticUnicodeOfLengthBetween(int minCodeUnits, int maxCodeUnits) {
     return RandomStrings.randomRealisticUnicodeOfLengthBetween(getRandom(),
         minCodeUnits, maxCodeUnits);
   }
   
-  /** @see StringGenerator#ofCodeUnitsLength(Random, int, int) */
+  /**
+   * @see RandomStrings#randomRealisticUnicodeOfLength
+   */
   public static String randomRealisticUnicodeOfLength(int codeUnits) {
     return RandomStrings.randomRealisticUnicodeOfLength(getRandom(), codeUnits);
   }
   
-  /** @see StringGenerator#ofCodePointsLength(Random, int, int) */
+  /**
+   * @see RandomStrings#randomRealisticUnicodeOfCodepointLengthBetween
+   */
   public static String randomRealisticUnicodeOfCodepointLengthBetween(
       int minCodePoints, int maxCodePoints) {
     return RandomStrings.randomRealisticUnicodeOfCodepointLengthBetween(
         getRandom(), minCodePoints, maxCodePoints);
   }
   
-  /** @see StringGenerator#ofCodePointsLength(Random, int, int) */
+  /**
+   * @see RandomStrings#randomRealisticUnicodeOfCodepointLength
+   */
   public static String randomRealisticUnicodeOfCodepointLength(int codePoints) {
     return RandomStrings.randomRealisticUnicodeOfCodepointLength(getRandom(),
         codePoints);
@@ -581,7 +730,7 @@ public class RandomizedTest extends Assert {
 
   /**
    * @param condition
-   *          If <code>false</code> an {@link InternalAssumptionViolatedException} is
+   *          If <code>false</code> an {@link AssumptionViolatedException} is
    *          thrown by this method and the test case (should be) ignored (or
    *          rather technically, flagged as a failure not passing a certain
    *          assumption). Tests that are assumption-failures do not break
@@ -592,7 +741,7 @@ public class RandomizedTest extends Assert {
   public static void assumeTrue(String message, boolean condition) {
     if (!condition) {
       // @see {@link Rants#RANT_2}.
-      throw new InternalAssumptionViolatedException(message);
+      throw new AssumptionViolatedException(message);
     }
   }
 
@@ -609,7 +758,7 @@ public class RandomizedTest extends Assert {
   public static void assumeNoException(String msg, Throwable t) {
     if (t != null) {
       // This does chain the exception as the cause.
-      throw new InternalAssumptionViolatedException(msg, t);
+      throw new AssumptionViolatedException(msg, t);
     }
   }
   
