@@ -29,11 +29,13 @@ import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
+import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
+import java.util.concurrent.locks.LockSupport;
 import java.util.logging.Logger;
 
 import org.junit.Test;
-import org.junit.internal.AssumptionViolatedException;
+import org.junit.AssumptionViolatedException;
 import org.junit.runner.Description;
 import org.junit.runner.Result;
 import org.junit.runner.notification.Failure;
@@ -45,6 +47,7 @@ import org.junit.runners.model.Statement;
 
 import com.carrotsearch.randomizedtesting.RandomizedRunner.TestCandidate;
 import com.carrotsearch.randomizedtesting.RandomizedRunner.UncaughtException;
+import com.carrotsearch.randomizedtesting.annotations.SuppressForbidden;
 import com.carrotsearch.randomizedtesting.annotations.ThreadLeakAction;
 import com.carrotsearch.randomizedtesting.annotations.ThreadLeakAction.Action;
 import com.carrotsearch.randomizedtesting.annotations.ThreadLeakFilters;
@@ -72,17 +75,17 @@ class ThreadLeakControl {
   private static class DefaultAnnotationValues {}
 
   /**
-   * Shared logger.
+   * Shared LOGGER.
    */
-  private final static Logger logger = RandomizedRunner.logger;
+  private final static Logger LOGGER = RandomizedRunner.logger;
 
-  /** 
+  /**
    * How many attempts to interrupt and then kill a runaway thread before giving up?
    */
   private final int killAttempts;
 
   /**
-   * How long to wait between attempts to kill a runaway thread (millis). 
+   * How long to wait between attempts to kill a runaway thread (millis).
    */
   private final int killWait;
 
@@ -97,7 +100,7 @@ class ThreadLeakControl {
   private final Set<Thread> expectedSuiteState;
 
   /**
-   * Atomic section for passing notifier events.  
+   * Atomic section for passing notifier events.
    */
   private final Object notifierLock = new Object();
 
@@ -110,7 +113,7 @@ class ThreadLeakControl {
    * Test timeout.
    */
   private TimeoutValue testTimeout;
-  
+
   /**
    * Suite timeout.
    */
@@ -120,7 +123,7 @@ class ThreadLeakControl {
    * Built-in filters.
    */
   private final List<ThreadFilter> builtinFilters;
-  
+
   /**
    * User filter (compound).
    */
@@ -183,7 +186,7 @@ class ThreadLeakControl {
         targetNotifier.fireTestFailure(failure);
       }
     }
-    
+
     @Override
     public void fireTestIgnored(Description description) {
       synchronized (notifierLock) {
@@ -258,7 +261,7 @@ class ThreadLeakControl {
       return this.t == t;
     }
   }
-  
+
   private static ThreadFilter or(final ThreadFilter... filters) {
     return new ThreadFilter() {
       @Override
@@ -293,7 +296,12 @@ class ThreadLeakControl {
       if (t.getName().equals("JFR request timer")) {
         return true;
       }
-      
+
+      // Explicit check for YourKit Java Profiler (YJP) agent thread.
+      if (t.getName().equals("YJPAgent-Telemetry")) {
+        return true;
+      }
+
       // J9 memory pool thread.
       if (t.getName().equals("MemoryPoolMXBean notification dispatcher")) {
         return true;
@@ -318,11 +326,11 @@ class ThreadLeakControl {
       Collections.reverse(stack);
 
       // Explicit check for GC$Daemon
-      if (stack.size() >= 1 && 
+      if (stack.size() >= 1 &&
           stack.get(0).getClassName().startsWith("sun.misc.GC$Daemon")) {
         return true;
       }
-      
+
       return false;
     }
   }
@@ -341,16 +349,16 @@ class ThreadLeakControl {
     suiteTimeout = new TimeoutValue(SYSPROP_TIMEOUT_SUITE(), DEFAULT_TIMEOUT_SUITE);
 
     builtinFilters = Arrays.asList(
-           new ThisThreadFilter(Thread.currentThread()),
-           new KnownSystemThread());
+        new ThisThreadFilter(Thread.currentThread()),
+        new KnownSystemThread());
 
     // Determine a set of expected threads up front (unfiltered).
     expectedSuiteState = Collections.unmodifiableSet(Threads.getAllThreads());
   }
 
   /**
-   * Runs a {@link Statement} and keeps any exception and 
-   * completion flag. 
+   * Runs a {@link Statement} and keeps any exception and
+   * completion flag.
    */
   private static class StatementRunner implements Runnable {
     private final Statement s;
@@ -383,7 +391,7 @@ class ThreadLeakControl {
   }
 
   /**
-   * A {@link Statement} for wrapping suite-level execution. 
+   * A {@link Statement} for wrapping suite-level execution.
    */
   Statement forSuite(final Statement s, final Description suiteDescription) {
     final Class<?> suiteClass = RandomizedContext.current().getTargetClass();
@@ -406,12 +414,13 @@ class ThreadLeakControl {
           if (timedOut) {
             // Mark as timed out so that we don't do any checks in any currently running test
             suiteTimedOut.set(true);
+
             // Flush streams so that we don't get warning outputs before sysout buffers.
-            System.out.flush();
-            System.err.flush();
+            flushStreams();
 
             // Emit a warning.
-            logger.warning("Suite execution timed out: " + suiteDescription + formatThreadStacksFull());
+            LOGGER.warning("Suite execution timed out: " + suiteDescription + formatThreadStacksFull());
+
             // mark subNotifier as dead (no longer passing events).
             subNotifier.pleaseStop();
           }
@@ -422,14 +431,14 @@ class ThreadLeakControl {
           if (subNotifier.testInProgress != null) {
             targetNotifier.fireTestFailure(
                 new Failure(subNotifier.testInProgress,
-                RandomizedRunner.augmentStackTrace(
-                    emptyStack(new Exception("Test abandoned because suite timeout was reached.")))));
+                    RandomizedRunner.augmentStackTrace(
+                        emptyStack(new Exception("Test abandoned because suite timeout was reached.")))));
             targetNotifier.fireTestFinished(subNotifier.testInProgress);
           }
 
           // throw suite failure (timeout).
           errors.add(RandomizedRunner.augmentStackTrace(
-              emptyStack(new Exception("Suite timeout exceeded (>= " + timeout + " msec)."))));        
+              emptyStack(new Exception("Suite timeout exceeded (>= " + timeout + " msec)."))));
         }
 
         final AnnotatedElement [] chain = { suiteClass, DefaultAnnotationValues.class };
@@ -440,11 +449,17 @@ class ThreadLeakControl {
 
         MultipleFailureException.assertEmpty(errors);
       }
+
+      @SuppressForbidden("Legitimate use of syserr.")
+      private void flushStreams() {
+        System.out.flush();
+        System.err.flush();
+      }
     };
   }
 
   /**
-   * A {@link Statement} for wrapping test-level execution. 
+   * A {@link Statement} for wrapping test-level execution.
    */
   Statement forTest(final Statement s, final TestCandidate c) {
     final int timeout = determineTimeout(c);
@@ -464,7 +479,7 @@ class ThreadLeakControl {
         }
 
         if (timedOut) {
-          logger.warning("Test execution timed out: " + c.description + formatThreadStacksFull());
+          LOGGER.warning("Test execution timed out: " + c.description + formatThreadStacksFull());
         }
 
         if (timedOut) {
@@ -472,8 +487,8 @@ class ThreadLeakControl {
               emptyStack(new Exception("Test timeout exceeded (>= " + timeout + " msec)."))));
         }
 
-        final AnnotatedElement [] chain = 
-          { c.method, c.instanceProvider.getTestClass(), DefaultAnnotationValues.class };
+        final AnnotatedElement [] chain =
+            { c.method, c.getTestClass(), DefaultAnnotationValues.class };
         List<Throwable> threadLeakErrors = timedOut ? new ArrayList<Throwable>() : errors;
         checkThreadLeaks(beforeTestState, threadLeakErrors, LifecycleScope.TEST, c.description, chain);
         processUncaught(errors, runner.handler.getUncaughtAndClear());
@@ -500,7 +515,7 @@ class ThreadLeakControl {
    * Instantiate a full set of {@link ThreadFilter}s for a suite.
    */
   private ThreadFilter instantiateFilters(List<Throwable> errors, Class<?> suiteClass) {
-    ThreadLeakFilters ann = 
+    ThreadLeakFilters ann =
         firstAnnotated(ThreadLeakFilters.class, suiteClass, DefaultAnnotationValues.class);
 
     final ArrayList<ThreadFilter> filters = new ArrayList<ThreadFilter>();
@@ -520,13 +535,13 @@ class ThreadLeakControl {
   }
 
   /**
-   * Clears a {@link Throwable}'s stack. 
+   * Clears a {@link Throwable}'s stack.
    */
   private static <T extends Throwable> T emptyStack(T t) {
     t.setStackTrace(new StackTraceElement [0]);
     return t;
   }
-  
+
   /**
    * Process uncaught exceptions.
    */
@@ -538,12 +553,12 @@ class ThreadLeakControl {
   }
 
   /**
-   * Perform a thread leak check at the given scope. 
+   * Perform a thread leak check at the given scope.
    */
   @SuppressWarnings("deprecation")
   protected void checkThreadLeaks(
       Set<Thread> expectedState,
-      List<Throwable> errors, 
+      List<Throwable> errors,
       LifecycleScope scope, Description description,
       AnnotatedElement... annotationChain)
   {
@@ -558,7 +573,7 @@ class ThreadLeakControl {
       return;
     }
 
-    // Check for the set of live threads, with optional lingering. 
+    // Check for the set of live threads, with optional lingering.
     int lingerTime = firstAnnotated(ThreadLeakLingering.class, annotationChain).linger();
     HashSet<Thread> threads = getThreads(suiteFilters);
     threads.removeAll(expectedState);
@@ -566,27 +581,27 @@ class ThreadLeakControl {
     if (lingerTime > 0 && !threads.isEmpty()) {
       final long deadline = System.currentTimeMillis() + lingerTime;
       try {
-        logger.warning("Will linger awaiting termination of " + threads.size() + " leaked thread(s).");
+        LOGGER.warning("Will linger awaiting termination of " + threads.size() + " leaked thread(s).");
         do {
           // Check every few hundred milliseconds until deadline occurs. We want to break out
           // sooner than the maximum lingerTime but there is no explicit even that
           // would wake us up, so poll periodically.
-          Thread.sleep(250);
+          Thread.sleep(100);
 
           threads = getThreads(suiteFilters);
           threads.removeAll(expectedState);
-          if (threads.isEmpty() || System.currentTimeMillis() > deadline) 
+          if (threads.isEmpty() || System.currentTimeMillis() > deadline)
             break;
         } while (true);
       } catch (InterruptedException e) {
-        logger.warning("Lingering interrupted.");
+        LOGGER.warning("Lingering interrupted.");
       }
     }
 
     if (threads.isEmpty()) {
       return;
     }
-    
+
     // Take one more snapshot, this time including stack traces (costly).
     HashMap<Thread,StackTraceElement[]> withTraces = getThreadsWithTraces(suiteFilters);
     withTraces.keySet().removeAll(expectedState);
@@ -610,7 +625,7 @@ class ThreadLeakControl {
     actions.addAll(Arrays.asList(firstAnnotated(ThreadLeakAction.class, annotationChain).value()));
 
     if (actions.contains(Action.WARN)) {
-      logger.severe(message.toString());
+      LOGGER.severe(message.toString());
     }
 
     Set<Thread> zombies = Collections.emptySet();
@@ -640,9 +655,9 @@ class ThreadLeakControl {
   private String formatThreadStacks(Map<Thread,StackTraceElement[]> threads) {
     StringBuilder message = new StringBuilder();
     int cnt = 1;
-    final Formatter f = new Formatter(message);
+    final Formatter f = new Formatter(message, Locale.ROOT);
     for (Map.Entry<Thread,StackTraceElement[]> e : threads.entrySet()) {
-      f.format(Locale.ENGLISH, "\n  %2d) %s", cnt++, Threads.threadName(e.getKey())).flush();
+      f.format(Locale.ROOT, "\n  %2d) %s", cnt++, Threads.threadName(e.getKey())).flush();
       if (e.getValue().length == 0) {
         message.append("\n        at (empty stack)");
       } else {
@@ -657,10 +672,10 @@ class ThreadLeakControl {
   /** Collect thread names. */
   private String threadNames(Collection<Thread> threads) {
     StringBuilder b = new StringBuilder();
-    final Formatter f = new Formatter(b);
+    final Formatter f = new Formatter(b, Locale.ROOT);
     int cnt = 1;
     for (Thread t : threads) {
-      f.format(Locale.ENGLISH, "\n  %2d) %s", cnt++, Threads.threadName(t));
+      f.format(Locale.ROOT, "\n  %2d) %s", cnt++, Threads.threadName(t));
     }
     return b.toString();
   }
@@ -680,7 +695,7 @@ class ThreadLeakControl {
     }
     return formatThreadStacks(getThreadsWithTraces());
   }
-  
+
   private static StackTraceElement[] getStackTrace(final Thread t) {
     return AccessController.doPrivileged(new PrivilegedAction<StackTraceElement[]>() {
       @Override
@@ -698,7 +713,7 @@ class ThreadLeakControl {
     final Set<Thread> threads = getThreads(filters);
     final HashMap<Thread,StackTraceElement[]> r = new HashMap<Thread,StackTraceElement[]>();
     for (Thread t : threads) {
-        r.put(t, getStackTrace(t));
+      r.put(t, getStackTrace(t));
     }
     return r;
   }
@@ -737,7 +752,7 @@ class ThreadLeakControl {
    * Attempt to interrupt all threads in the given set.
    */
   private Set<Thread> tryToInterruptAll(List<Throwable> errors, Set<Thread> threads) {
-    logger.info("Starting to interrupt leaked threads:" + threadNames(threads));
+    LOGGER.info("Starting to interrupt leaked threads:" + threadNames(threads));
 
     // stop reporting uncaught exceptions.
     runner.handler.stopReporting();
@@ -745,7 +760,7 @@ class ThreadLeakControl {
       // This means we have an unknown ordering of interrupt calls but
       // there is very little we can do about it, really.
       final HashSet<Thread> ordered = new HashSet<Thread>(threads);
-  
+
       int interruptAttempts = this.killAttempts;
       int interruptWait = this.killWait;
       boolean allDead;
@@ -757,7 +772,7 @@ class ThreadLeakControl {
           for (Thread t : ordered) {
             t.interrupt();
           }
-          
+
           // Maximum wait time. Progress through the threads, trying to join but
           // decrease the join time each time.
           long waitDeadline = System.currentTimeMillis() + interruptWait;
@@ -765,7 +780,7 @@ class ThreadLeakControl {
             final Thread t = i.next();
             if (t.isAlive()) {
               allDead = false;
-              t.join(Math.max(1, waitDeadline - System.currentTimeMillis()));
+              join(t, Math.max(1, waitDeadline - System.currentTimeMillis()), Thread::sleep);
             } else {
               i.remove();
             }
@@ -773,9 +788,9 @@ class ThreadLeakControl {
         } catch (InterruptedException e) {
           interruptAttempts = 0;
         }
-      } while (!allDead && --interruptAttempts >= 0);
+      } while (!allDead && --interruptAttempts > 0);
       Thread.currentThread().setPriority(restorePriority);
-  
+
       // Check after the last join.
       HashMap<Thread,StackTraceElement[]> zombies = new HashMap<Thread,StackTraceElement[]>();
       for (Thread t : ordered) {
@@ -783,12 +798,12 @@ class ThreadLeakControl {
           zombies.put(t, getStackTrace(t));
         }
       }
-  
+
       if (zombies.isEmpty()) {
-        logger.info("All leaked threads terminated.");
+        LOGGER.info("All leaked threads terminated.");
       } else {
         String message = "There are still zombie threads that couldn't be terminated:" + formatThreadStacks(zombies);
-        logger.severe(message);
+        LOGGER.severe(message);
         errors.add(RandomizedRunner.augmentStackTrace(
             emptyStack(new ThreadLeakError(message.toString()))));
       }
@@ -802,16 +817,32 @@ class ThreadLeakControl {
   /**
    * Fork or not depending on the timeout value.
    */
-  boolean forkTimeoutingTask(StatementRunner r, int timeout, List<Throwable> errors) 
+  boolean forkTimeoutingTask(StatementRunner r, int timeout, List<Throwable> errors)
       throws InterruptedException
   {
     if (timeout == 0) {
       r.run();
     } else {
-      Thread t = new Thread(r, Thread.currentThread().getName() + "-worker");
+      final Thread owner = Thread.currentThread();
+      final AtomicBoolean done = new AtomicBoolean();
+
+      Thread t = new Thread(() -> {
+        try {
+          r.run();
+        } finally {
+          done.set(true);
+          LockSupport.unpark(owner);
+        }
+      }, Thread.currentThread().getName() + "-worker");
       RandomizedContext.cloneFor(t);
+
       t.start();
-      t.join(timeout);
+      join(t, timeout, (millis) -> {
+        LockSupport.parkNanos(TimeUnit.MILLISECONDS.toNanos(millis));
+        if (done.get()) {
+          t.join();
+        }
+      });
     }
 
     final boolean timedOut = !r.completed;
@@ -819,6 +850,29 @@ class ThreadLeakControl {
     return timedOut;
   }
 
+  static interface AwaitCond {
+    void await(long millis) throws InterruptedException;
+  }
+
+  static void join(Thread t, long millis, AwaitCond cond) throws InterruptedException {
+    if (millis <= 0) {
+      throw new IllegalArgumentException("Timeout must be positive: " + millis);
+    }
+
+    long deadline = System.currentTimeMillis() + millis;
+    while (t.isAlive()) {
+      long delay = deadline - System.currentTimeMillis();
+      if (delay > 0) {
+        // Don't wait longer than a few millis, then recheck condition.
+        // We use sleep because Thread.join() is synchronized and this thread may
+        // get stuck on getting the monitor for an indefinite amount of time.
+        cond.await(Math.min(250, delay));
+      } else {
+        break;
+      }
+    }
+  }
+
   boolean isTimedOut() {
     return suiteTimedOut.get();
   }
@@ -834,7 +888,7 @@ class ThreadLeakControl {
 
   /**
    * Determine timeout for a suite.
-   * 
+   *
    * @return Returns timeout in milliseconds or 0 if the test should run until
    *         finished (possibly blocking forever).
    */
@@ -845,14 +899,14 @@ class ThreadLeakControl {
 
   /**
    * Determine timeout for a single test method (candidate).
-   *  
+   *
    * @return Returns timeout in milliseconds or 0 if the test should run until
    *         finished (possibly blocking forever).
    */
   private int determineTimeout(TestCandidate c) {
     Integer timeout = null;
 
-    Timeout timeoutAnn = c.instanceProvider.getTestClass().getAnnotation(Timeout.class);
+    Timeout timeoutAnn = c.getTestClass().getAnnotation(Timeout.class);
     if (timeoutAnn != null) {
       timeout = (int) Math.min(Integer.MAX_VALUE, timeoutAnn.millis());
     }
@@ -870,7 +924,7 @@ class ThreadLeakControl {
     }
 
     return testTimeout.getTimeout(timeout);
-  }  
+  }
 
   /**
    * Returns an annotation's instance declared on any annotated element (first one wins)
