@@ -1,24 +1,27 @@
 package com.carrotsearch.randomizedtesting;
 
 import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertTrue;
 
 import java.util.HashMap;
 import java.util.Map;
 
+import org.junit.BeforeClass;
 import org.junit.Test;
-import org.junit.runner.JUnitCore;
-import org.junit.runner.Result;
 import org.junit.runner.RunWith;
 
 import com.carrotsearch.randomizedtesting.annotations.Seed;
 
-public class TestSeedRepeatable {
+public class TestSeedRepeatable extends WithNestedTestClass {
   @Seed("deadbeef")
   @RunWith(RandomizedRunner.class)
   public static class Nested {
     static Map<String, Object> seeds = new HashMap<String, Object>();
 
+    @BeforeClass
+    public static void nested() {
+      assumeRunningNested();
+    }
+    
     @Test
     public void testMethod1() {
       checkSeed("method1");
@@ -45,11 +48,8 @@ public class TestSeedRepeatable {
   @Test
   public void testSameMethodRandomnessWithFixedRunner() {
     Nested.seeds.clear();
-    Result result = JUnitCore.runClasses(Nested.class);
-    assertTrue(result.getFailures().toString(), result.wasSuccessful());
-    result = JUnitCore.runClasses(Nested.class);
-    assertTrue(result.wasSuccessful());
-    result = JUnitCore.runClasses(Nested.class);    
-    assertTrue(result.wasSuccessful());
+    checkTestsOutput(2, 0, 0, 0, Nested.class);
+    checkTestsOutput(2, 0, 0, 0, Nested.class);
+    checkTestsOutput(2, 0, 0, 0, Nested.class);
   }  
 }
