@@ -1,8 +1,6 @@
 package com.carrotsearch.randomizedtesting.timeouts;
 
 import org.junit.Test;
-import org.junit.runner.JUnitCore;
-import org.junit.runner.Result;
 
 import com.carrotsearch.randomizedtesting.RandomizedTest;
 import com.carrotsearch.randomizedtesting.Utils;
@@ -30,7 +28,7 @@ public class Test002TimeoutMethod extends WithNestedTestClass {
       }
     };
 
-    Result r = JUnitCore.runClasses(Nested.class);
+    FullResult r = runTests(Nested.class);
     Utils.assertFailureWithMessage(r, "Test timeout exceeded");
     Utils.assertFailuresContainSeeds(r);
   }
