@@ -1,10 +1,7 @@
 package com.carrotsearch.randomizedtesting.timeouts;
 
-import junit.framework.Assert;
-
 import org.junit.Test;
-import org.junit.runner.JUnitCore;
-import org.junit.runner.Result;
+import org.junit.Assert;
 
 import com.carrotsearch.randomizedtesting.RandomizedTest;
 import com.carrotsearch.randomizedtesting.WithNestedTestClass;
@@ -31,10 +28,11 @@ public class Test014Timeout extends WithNestedTestClass {
 
   @Test
   public void testTimeoutInTestAnnotation() {
-    Result result = JUnitCore.runClasses(Nested.class);
+    FullResult result = runTests(Nested.class);
 
     Assert.assertEquals(0, result.getIgnoreCount());
     Assert.assertEquals(2, result.getRunCount());
     Assert.assertEquals(2, result.getFailureCount());
+    Assert.assertEquals(0, result.getAssumptionIgnored());
   }
 }
